@@ -92,7 +92,7 @@ pub type DefaultHashBuilder = std::collections::hash_map::RandomState;
 ///
 /// impl Weighter<u64, String> for StringWeighter {
 ///     fn weight(&self, _key: &u64, val: &String) -> u64 {
-///         // Be cautions out about zero weights!
+///         // Be cautious out about zero weights!
 ///         val.len() as u64
 ///     }
 /// }
@@ -289,7 +289,7 @@ mod tests {
             let cache = sync::Cache::<(u64, u64), u64>::new(100);
             const THREADS: usize = 100;
             let wg = std::sync::Barrier::new(THREADS);
-            let solve_at = rand::thread_rng().gen_range(0..THREADS);
+            let solve_at = rand::rng().random_range(0..THREADS);
             std::thread::scope(|s| {
                 for _ in 0..THREADS {
                     s.spawn(|| {
@@ -371,6 +371,78 @@ mod tests {
         assert_eq!(result, Ok(4));
     }
 
+    #[test]
+    fn test_retain_unsync() {
+        let mut cache = unsync::Cache::<u64, u64>::new(100);
+        let ranges = 0..10;
+        for i in ranges.clone() {
+            let guard = cache.get_ref_or_guard(&i).unwrap_err();
+            guard.insert(i);
+            assert_eq!(cache.get_ref_or_guard(&i).ok().copied(), Some(i));
+        }
+        let small = 3;
+        cache.retain(|&key, &val| val > small && key > small);
+        for i in ranges.clone() {
+            let actual = cache.get(&i);
+            if i > small {
+                assert!(actual.is_some());
+                assert_eq!(*actual.unwrap(), i);
+            } else {
+                assert!(actual.is_none());
+            }
+        }
+        let big = 7;
+        cache.retain(|&key, &val| val < big && key < big);
+        for i in ranges {
+            let actual = cache.get(&i);
+            if i > small && i < big {
+                assert!(actual.is_some());
+                assert_eq!(*actual.unwrap(), i);
+            } else {
+                assert!(actual.is_none());
+            }
+        }
+    }
+
+    #[tokio::test]
+    async fn test_retain_sync() {
+        use crate::sync::*;
+        let cache = Cache::<u64, u64>::new(100);
+        let ranges = 0..10;
+        for i in ranges.clone() {
+            let GuardResult::Guard(guard) = cache.get_value_or_guard(&i, None) else {
+                panic!();
+            };
+            guard.insert(i).unwrap();
+            let GuardResult::Value(v) = cache.get_value_or_guard(&i, None) else {
+                panic!();
+            };
+            assert_eq!(v, i);
+        }
+        let small = 4;
+        cache.retain(|&key, &val| val > small && key > small);
+        for i in ranges.clone() {
+            let actual = cache.get(&i);
+            if i > small {
+                assert!(actual.is_some());
+                assert_eq!(actual.unwrap(), i);
+            } else {
+                assert!(actual.is_none());
+            }
+        }
+        let big = 8;
+        cache.retain(|&key, &val| val < big && key < big);
+        for i in ranges {
+            let actual = cache.get(&i);
+            if i > small && i < big {
+                assert!(actual.is_some());
+                assert_eq!(actual.unwrap(), i);
+            } else {
+                assert!(actual.is_none());
+            }
+        }
+    }
+
     #[test]
     #[cfg_attr(miri, ignore)]
     fn test_value_or_guard() {
@@ -382,7 +454,7 @@ mod tests {
             let cache = sync::Cache::<(u64, u64), u64>::new(100);
             const THREADS: usize = 100;
             let wg = std::sync::Barrier::new(THREADS);
-            let solve_at = rand::thread_rng().gen_range(0..THREADS);
+            let solve_at = rand::rng().random_range(0..THREADS);
             std::thread::scope(|s| {
                 for _ in 0..THREADS {
                     s.spawn(|| {
@@ -419,7 +491,7 @@ mod tests {
             let cache = Arc::new(sync::Cache::<(u64, u64), u64>::new(100));
             const TASKS: usize = 100;
             let wg = Arc::new(tokio::sync::Barrier::new(TASKS));
-            let solve_at = rand::thread_rng().gen_range(0..TASKS);
+            let solve_at = rand::rng().random_range(0..TASKS);
             let mut tasks = Vec::new();
             for _ in 0..TASKS {
                 let cache = cache.clone();
@@ -462,7 +534,7 @@ mod tests {
             let cache = Arc::new(sync::Cache::<(u64, u64), u64>::new(100));
             const TASKS: usize = 100;
             let wg = Arc::new(tokio::sync::Barrier::new(TASKS));
-            let solve_at = rand::thread_rng().gen_range(0..TASKS);
+            let solve_at = rand::rng().random_range(0..TASKS);
             let mut tasks = Vec::new();
             for _ in 0..TASKS {
                 let cache = cache.clone();
