@@ -3,7 +3,7 @@ use std::{
     mem::{self, MaybeUninit},
 };
 
-use hashbrown::raw::RawTable;
+use hashbrown::HashTable;
 
 use crate::{
     linked_slab::{LinkedSlab, Token},
@@ -76,7 +76,7 @@ pub struct CacheShard<Key, Val, We, B, L, Plh> {
     hash_builder: B,
     /// Map to an entry in the `entries` slab.
     /// Note that the actual key/value/hash are not stored in the map but in the slab.
-    map: RawTable<Token>,
+    map: HashTable<Token>,
     /// Slab holding entries
     entries: LinkedSlab<Entry<Key, Val, Plh>>,
     /// Head of cold list, containing Cold entries.
@@ -174,13 +174,15 @@ macro_rules! record_miss_mut {
 
 impl<Key, Val, We, B, L, Plh: SharedPlaceholder> CacheShard<Key, Val, We, B, L, Plh> {
     pub fn remove_placeholder(&mut self, placeholder: &Plh) {
-        self.map.remove_entry(placeholder.hash(), |&idx| {
+        if let Ok(entry) = self.map.find_entry(placeholder.hash(), |&idx| {
             if idx != placeholder.idx() {
                 return false;
             }
             let (entry, _) = self.entries.get(idx).unwrap();
             matches!(entry, Entry::Placeholder(Placeholder { shared, .. }) if shared.same_as(placeholder))
-        });
+        }) {
+            entry.remove();
+        }
     }
 
     #[cold]
@@ -204,6 +206,69 @@ impl<Key, Val, We, B, L, Plh: SharedPlaceholder> CacheShard<Key, Val, We, B, L,
     }
 }
 
+impl<Key, Val, We, B, L, Plh> CacheShard<Key, Val, We, B, L, Plh> {
+    pub fn weight(&self) -> u64 {
+        self.weight_hot + self.weight_cold
+    }
+
+    pub fn len(&self) -> usize {
+        self.num_hot + self.num_cold
+    }
+
+    pub fn capacity(&self) -> u64 {
+        self.weight_capacity
+    }
+
+    #[cfg(feature = "stats")]
+    pub fn hits(&self) -> u64 {
+        self.hits.load(atomic::Ordering::Relaxed)
+    }
+
+    #[cfg(feature = "stats")]
+    pub fn misses(&self) -> u64 {
+        self.misses.load(atomic::Ordering::Relaxed)
+    }
+
+    pub fn clear(&mut self) {
+        let _ = self.drain();
+    }
+
+    pub fn drain(&mut self) -> impl Iterator<Item = (Key, Val)> + '_ {
+        self.cold_head = None;
+        self.hot_head = None;
+        self.ghost_head = None;
+        self.num_hot = 0;
+        self.num_cold = 0;
+        self.num_non_resident = 0;
+        self.weight_hot = 0;
+        self.weight_cold = 0;
+        self.map.clear();
+        self.entries.drain().filter_map(|i| match i {
+            Entry::Resident(r) => Some((r.key, r.value)),
+            Entry::Placeholder(_) | Entry::Ghost(_) => None,
+        })
+    }
+
+    pub fn iter(&self) -> impl Iterator<Item = (&'_ Key, &'_ Val)> + '_ {
+        self.entries.iter().filter_map(|i| match i {
+            Entry::Resident(r) => Some((&r.key, &r.value)),
+            Entry::Placeholder(_) | Entry::Ghost(_) => None,
+        })
+    }
+
+    pub fn iter_from(
+        &self,
+        continuation: Option<Token>,
+    ) -> impl Iterator<Item = (Token, &'_ Key, &'_ Val)> + '_ {
+        self.entries
+            .iter_from(continuation)
+            .filter_map(|(token, i)| match i {
+                Entry::Resident(r) => Some((token, &r.key, &r.value)),
+                Entry::Placeholder(_) | Entry::Ghost(_) => None,
+            })
+    }
+}
+
 impl<
         Key: Eq + Hash,
         Val,
@@ -226,7 +291,7 @@ impl<
         let capacity_non_resident = (estimated_items_capacity as f64 * ghost_allocation) as usize;
         Self {
             hash_builder,
-            map: RawTable::with_capacity(0),
+            map: HashTable::with_capacity(0),
             entries: LinkedSlab::with_capacity(0),
             weight_capacity,
             #[cfg(feature = "stats")]
@@ -330,53 +395,31 @@ impl<
         })
     }
 
-    pub fn weight(&self) -> u64 {
-        self.weight_hot + self.weight_cold
-    }
-
-    pub fn len(&self) -> usize {
-        self.num_hot + self.num_cold
-    }
-
-    pub fn capacity(&self) -> u64 {
-        self.weight_capacity
-    }
-
-    #[cfg(feature = "stats")]
-    pub fn hits(&self) -> u64 {
-        self.hits.load(atomic::Ordering::Relaxed)
-    }
-
-    #[cfg(feature = "stats")]
-    pub fn misses(&self) -> u64 {
-        self.misses.load(atomic::Ordering::Relaxed)
-    }
-
-    pub fn clear(&mut self) {
-        let _ = self.drain();
-    }
-
-    pub fn drain(&mut self) -> impl Iterator<Item = (Key, Val)> + '_ {
-        self.cold_head = None;
-        self.hot_head = None;
-        self.ghost_head = None;
-        self.num_hot = 0;
-        self.num_cold = 0;
-        self.num_non_resident = 0;
-        self.weight_hot = 0;
-        self.weight_cold = 0;
-        self.map.clear();
-        self.entries.drain().filter_map(|i| match i {
-            Entry::Resident(r) => Some((r.key, r.value)),
-            Entry::Placeholder(_) | Entry::Ghost(_) => None,
-        })
-    }
-
-    pub fn iter(&self) -> impl Iterator<Item = (&'_ Key, &'_ Val)> + '_ {
-        self.entries.iter().filter_map(|i| match i {
-            Entry::Resident(r) => Some((&r.key, &r.value)),
-            Entry::Placeholder(_) | Entry::Ghost(_) => None,
-        })
+    pub fn retain<F>(&mut self, f: F)
+    where
+        F: Fn(&Key, &Val) -> bool,
+    {
+        let retained_tokens = self
+            .map
+            .iter()
+            .filter_map(|&idx| match self.entries.get(idx) {
+                Some((entry, _idx)) => match entry {
+                    Entry::Resident(r) => {
+                        if !f(&r.key, &r.value) {
+                            let hash = self.hash(&r.key);
+                            Some((idx, hash))
+                        } else {
+                            None
+                        }
+                    }
+                    Entry::Placeholder(_) | Entry::Ghost(_) => None,
+                },
+                None => None,
+            })
+            .collect::<Vec<_>>();
+        for (idx, hash) in retained_tokens {
+            self.remove_internal(hash, idx);
+        }
     }
 
     #[inline]
@@ -400,32 +443,24 @@ impl<
     where
         Q: Hash + Equivalent<Key> + ?Sized,
     {
-        // Safety for `RawTable::iter_hash` and `Bucket::as_ref`:
-        // * Their outputs do not outlive their HashBrown:
-        // The HashBrown instance is alive for the entirety of the function and
-        // the content references never leave the function.
-        // * HashBrown can't be mutated while being iterated with iter_hash:
-        // The HashBrown instance isn't mutated in this method.
-        unsafe {
-            let mut hash_match = None;
-            for bucket in self.map.iter_hash(hash) {
-                let idx = *bucket.as_ref();
-                let (entry, _) = self.entries.get(idx).unwrap();
-                match entry {
-                    Entry::Resident(Resident { key, .. })
-                    | Entry::Placeholder(Placeholder { key, .. })
-                        if k.equivalent(key) =>
-                    {
-                        return Some(idx);
-                    }
-                    Entry::Ghost(non_resident_hash) if *non_resident_hash == hash => {
-                        hash_match = Some(idx);
-                    }
-                    _ => (),
+        let mut hash_match = None;
+        for bucket in self.map.iter_hash(hash) {
+            let idx = *bucket;
+            let (entry, _) = self.entries.get(idx).unwrap();
+            match entry {
+                Entry::Resident(Resident { key, .. })
+                | Entry::Placeholder(Placeholder { key, .. })
+                    if k.equivalent(key) =>
+                {
+                    return Some(idx);
                 }
+                Entry::Ghost(non_resident_hash) if *non_resident_hash == hash => {
+                    hash_match = Some(idx);
+                }
+                _ => (),
             }
-            hash_match
         }
+        hash_match
     }
 
     #[inline]
@@ -435,7 +470,7 @@ impl<
     {
         let mut resident = MaybeUninit::uninit();
         self.map
-            .get(hash, |&idx| {
+            .find(hash, |&idx| {
                 let (entry, _) = self.entries.get(idx).unwrap();
                 match entry {
                     Entry::Resident(r) if k.equivalent(&r.key) => {
@@ -452,6 +487,18 @@ impl<
             })
     }
 
+    pub fn contains<Q>(&self, hash: u64, key: &Q) -> bool
+    where
+        Q: Hash + Equivalent<Key> + ?Sized,
+    {
+        self.map
+            .find(hash, |&idx| {
+                let (entry, _) = self.entries.get(idx).unwrap();
+                matches!(entry, Entry::Resident(r) if key.equivalent(&r.key))
+            })
+            .is_some()
+    }
+
     pub fn get<Q>(&self, hash: u64, key: &Q) -> Option<&Val>
     where
         Q: Hash + Equivalent<Key> + ?Sized,
@@ -545,6 +592,28 @@ impl<
         self.remove_internal(hash, idx)
     }
 
+    pub fn remove_token(&mut self, token: Token) -> Option<(Key, Val)> {
+        let Some((Entry::Resident(resident), _)) = self.entries.get(token) else {
+            return None;
+        };
+        let hash = Self::hash_static(&self.hash_builder, &resident.key);
+        self.remove_internal(hash, token)
+    }
+
+    pub fn remove_next(&mut self, continuation: Option<Token>) -> Option<(Token, Key, Val)> {
+        let (token, key, _) = self
+            .entries
+            .iter_from(continuation)
+            .filter_map(|(token, i)| match i {
+                Entry::Resident(r) => Some((token, &r.key, &r.value)),
+                Entry::Placeholder(_) | Entry::Ghost(_) => None,
+            })
+            .next()?;
+        let hash = Self::hash_static(&self.hash_builder, key);
+        self.remove_internal(hash, token)
+            .map(|(k, v)| (token, k, v))
+    }
+
     fn remove_internal(&mut self, hash: u64, idx: Token) -> Option<(Key, Val)> {
         self.map_remove(hash, idx);
         let mut result = None;
@@ -803,7 +872,7 @@ impl<
 
     #[inline]
     fn map_insert(&mut self, hash: u64, idx: Token) {
-        self.map.insert(hash, idx, |&i| {
+        self.map.insert_unique(hash, idx, |&i| {
             let (entry, _) = self.entries.get(i).unwrap();
             match entry {
                 Entry::Resident(Resident { key, .. })
@@ -817,8 +886,12 @@ impl<
 
     #[inline]
     fn map_remove(&mut self, hash: u64, idx: Token) {
-        let removed = self.map.erase_entry(hash, |&i| i == idx);
-        debug_assert!(removed);
+        if let Ok(entry) = self.map.find_entry(hash, |&i| i == idx) {
+            entry.remove();
+            return;
+        }
+        #[cfg(debug_assertions)]
+        panic!("key not found");
     }
 
     pub fn replace_placeholder(
@@ -1037,8 +1110,8 @@ pub struct RefMut<'cache, Key, Val, We: Weighter<Key, Val>, B, L, Plh: SharedPla
     old_weight: u64,
 }
 
-impl<'cache, Key, Val, We: Weighter<Key, Val>, B, L, Plh: SharedPlaceholder>
-    RefMut<'cache, Key, Val, We, B, L, Plh>
+impl<Key, Val, We: Weighter<Key, Val>, B, L, Plh: SharedPlaceholder>
+    RefMut<'_, Key, Val, We, B, L, Plh>
 {
     pub(crate) fn pair(&self) -> (&Key, &Val) {
         // Safety: RefMut was constructed correctly from a Resident entry in get_mut or peek_token_mut
@@ -1069,8 +1142,8 @@ impl<'cache, Key, Val, We: Weighter<Key, Val>, B, L, Plh: SharedPlaceholder>
     }
 }
 
-impl<'cache, Key, Val, We: Weighter<Key, Val>, B, L, Plh: SharedPlaceholder> Drop
-    for RefMut<'cache, Key, Val, We, B, L, Plh>
+impl<Key, Val, We: Weighter<Key, Val>, B, L, Plh: SharedPlaceholder> Drop
+    for RefMut<'_, Key, Val, We, B, L, Plh>
 {
     #[inline]
     fn drop(&mut self) {
