@@ -5,6 +5,7 @@ use std::{
 };
 
 use crate::{
+    linked_slab::Token,
     options::{Options, OptionsBuilder},
     shard::{CacheShard, InsertStrategy},
     shim::rw_lock::RwLock,
@@ -14,7 +15,11 @@ use crate::{
 
 pub use crate::sync_placeholder::{GuardResult, JoinFuture, PlaceholderGuard};
 
-/// A concurrent cache.
+/// A concurrent cache
+///
+/// The concurrent cache is internally composed of equally sized shards, each of which is independently
+/// synchronized. This allows for low contention when multiple threads are accessing the cache but limits the
+/// maximum weight capacity of each shard.
 ///
 /// # Value
 /// Cache values are cloned when fetched. Users should wrap their values with `Arc<_>`
@@ -158,7 +163,7 @@ impl<
 
     /// Returns whether the cache is empty
     pub fn is_empty(&self) -> bool {
-        self.shards.iter().any(|s| s.read().len() == 0)
+        self.shards.iter().all(|s| s.read().len() == 0)
     }
 
     /// Returns the number of cached items
@@ -171,11 +176,23 @@ impl<
         self.shards.iter().map(|s| s.read().weight()).sum()
     }
 
-    /// Returns the maximum weight of cached items
+    /// Returns the _total_ maximum weight capacity of cached items.
+    /// Note that the cache may be composed of multiple shards and each shard has its own maximum weight capacity,
+    /// see [`Self::shard_capacity`].
     pub fn capacity(&self) -> u64 {
         self.shards.iter().map(|s| s.read().capacity()).sum()
     }
 
+    /// Returns the maximum weight capacity of each shard.
+    pub fn shard_capacity(&self) -> u64 {
+        self.shards[0].read().capacity()
+    }
+
+    /// Returns the number of shards.
+    pub fn num_shards(&self) -> usize {
+        self.shards.len()
+    }
+
     /// Returns the number of misses
     #[cfg(feature = "stats")]
     pub fn misses(&self) -> u64 {
@@ -218,6 +235,15 @@ impl<
         }
     }
 
+    /// Check if a key exist in the cache.
+    pub fn contains_key<Q>(&self, key: &Q) -> bool
+    where
+        Q: Hash + Equivalent<Key> + ?Sized,
+    {
+        self.shard_for(key)
+            .is_some_and(|(shard, hash)| shard.read().contains(hash, key))
+    }
+
     /// Fetches an item from the cache whose key is `key`.
     pub fn get<Q>(&self, key: &Q) -> Option<Val>
     where
@@ -278,6 +304,18 @@ impl<
         Ok(lcs)
     }
 
+    /// Retains only the items specified by the predicate.
+    /// In other words, remove all items for which `f(&key, &value)` returns `false`. The
+    /// elements are visited in arbitrary order.
+    pub fn retain<F>(&self, f: F)
+    where
+        F: Fn(&Key, &Val) -> bool,
+    {
+        for s in self.shards.iter() {
+            s.write().retain(&f);
+        }
+    }
+
     /// Inserts an item in the cache with key `key`.
     pub fn insert(&self, key: Key, value: Val) {
         let lcs = self.insert_with_lifecycle(key, value);
@@ -303,6 +341,40 @@ impl<
         }
     }
 
+    /// Iterates over the items in the cache returning cloned key value pairs.
+    ///
+    /// The iterator is guaranteed to yield all items in the cache at the time of creation
+    /// provided that they are not removed or evicted from the cache while iterating.
+    /// The iterator may also yield items added to the cache after the iterator is created.
+    pub fn iter(&self) -> Iter<'_, Key, Val, We, B, L>
+    where
+        Key: Clone,
+    {
+        Iter {
+            shards: &self.shards,
+            current_shard: 0,
+            last: None,
+        }
+    }
+
+    /// Drains items from the cache.
+    ///
+    /// The iterator is guaranteed to drain all items in the cache at the time of creation
+    /// provided that they are not removed or evicted from the cache while draining.
+    /// The iterator may also drain items added to the cache after the iterator is created.
+    /// Due to the above, the cache may not be empty after the iterator is fully consumed
+    /// if items are added to the cache while draining.
+    ///
+    /// Note that dropping the iterator will _not_ finish the draining process, unlike other
+    /// drain methods.
+    pub fn drain(&self) -> Drain<'_, Key, Val, We, B, L> {
+        Drain {
+            shards: &self.shards,
+            current_shard: 0,
+            last: None,
+        }
+    }
+
     /// Gets an item from the cache with key `key` .
     ///
     /// If the corresponding value isn't present in the cache, this functions returns a guard
@@ -312,11 +384,11 @@ impl<
     /// is dropped or the value is inserted.
     ///
     /// A `None` `timeout` means waiting forever.
-    pub fn get_value_or_guard<'a, Q>(
-        &'a self,
+    pub fn get_value_or_guard<Q>(
+        &self,
         key: &Q,
         timeout: Option<Duration>,
-    ) -> GuardResult<'a, Key, Val, We, B, L>
+    ) -> GuardResult<'_, Key, Val, We, B, L>
     where
         Q: Hash + Equivalent<Key> + ToOwned<Owned = Key> + ?Sized,
     {
@@ -371,7 +443,7 @@ impl<
     }
 
     /// Gets or inserts an item in the cache with key `key`.
-    pub async fn get_or_insert_async<'a, Q, E>(
+    pub async fn get_or_insert_async<Q, E>(
         &self,
         key: &Q,
         with: impl Future<Output = Result<Val, E>>,
@@ -396,6 +468,82 @@ impl<Key, Val, We, B, L> std::fmt::Debug for Cache<Key, Val, We, B, L> {
     }
 }
 
+/// Iterator over the items in the cache.
+///
+/// See [`Cache::iter`] for more details.
+pub struct Iter<'a, Key, Val, We, B, L> {
+    shards: &'a [RwLock<CacheShard<Key, Val, We, B, L, SharedPlaceholder<Val>>>],
+    current_shard: usize,
+    last: Option<Token>,
+}
+
+impl<Key, Val, We, B, L> Iterator for Iter<'_, Key, Val, We, B, L>
+where
+    Key: Clone,
+    Val: Clone,
+{
+    type Item = (Key, Val);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        while self.current_shard < self.shards.len() {
+            let shard = &self.shards[self.current_shard];
+            let lock = shard.read();
+            if let Some((new_last, key, val)) = lock.iter_from(self.last).next() {
+                self.last = Some(new_last);
+                return Some((key.clone(), val.clone()));
+            }
+            self.last = None;
+            self.current_shard += 1;
+        }
+        None
+    }
+}
+
+impl<Key, Val, We, B, L> std::fmt::Debug for Iter<'_, Key, Val, We, B, L> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Iter").finish_non_exhaustive()
+    }
+}
+
+/// Draining iterator for the items in the cache.
+///
+/// See [`Cache::drain`] for more details.
+pub struct Drain<'a, Key, Val, We, B, L> {
+    shards: &'a [RwLock<CacheShard<Key, Val, We, B, L, SharedPlaceholder<Val>>>],
+    current_shard: usize,
+    last: Option<Token>,
+}
+
+impl<Key, Val, We, B, L> Iterator for Drain<'_, Key, Val, We, B, L>
+where
+    Key: Hash + Eq,
+    We: Weighter<Key, Val>,
+    B: BuildHasher,
+    L: Lifecycle<Key, Val>,
+{
+    type Item = (Key, Val);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        while self.current_shard < self.shards.len() {
+            let shard = &self.shards[self.current_shard];
+            let mut lock = shard.write();
+            if let Some((new_last, key, value)) = lock.remove_next(self.last) {
+                self.last = Some(new_last);
+                return Some((key, value));
+            }
+            self.last = None;
+            self.current_shard += 1;
+        }
+        None
+    }
+}
+
+impl<Key, Val, We, B, L> std::fmt::Debug for Drain<'_, Key, Val, We, B, L> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Drain").finish_non_exhaustive()
+    }
+}
+
 /// Default `Lifecycle` for a sync cache.
 ///
 /// Temporally stashes one evicted item for dropping outside the cache locks.
@@ -485,4 +633,61 @@ mod tests {
             t.join().unwrap();
         }
     }
+
+    #[test]
+    fn test_iter() {
+        let capacity = if cfg!(miri) { 100 } else { 100000 };
+        let options = OptionsBuilder::new()
+            .estimated_items_capacity(capacity)
+            .weight_capacity(capacity as u64)
+            .shards(2)
+            .build()
+            .unwrap();
+        let cache = Cache::with_options(
+            options,
+            UnitWeighter,
+            DefaultHashBuilder::default(),
+            DefaultLifecycle::default(),
+        );
+        let items = capacity / 2;
+        for i in 0..items {
+            cache.insert(i, i);
+        }
+        assert_eq!(cache.len(), items);
+        let mut iter_collected = cache.iter().collect::<Vec<_>>();
+        assert_eq!(iter_collected.len(), items);
+        iter_collected.sort();
+        for (i, v) in iter_collected.into_iter().enumerate() {
+            assert_eq!((i, i), v);
+        }
+    }
+
+    #[test]
+    fn test_drain() {
+        let capacity = if cfg!(miri) { 100 } else { 100000 };
+        let options = OptionsBuilder::new()
+            .estimated_items_capacity(capacity)
+            .weight_capacity(capacity as u64)
+            .shards(2)
+            .build()
+            .unwrap();
+        let cache = Cache::with_options(
+            options,
+            UnitWeighter,
+            DefaultHashBuilder::default(),
+            DefaultLifecycle::default(),
+        );
+        let items = capacity / 2;
+        for i in 0..items {
+            cache.insert(i, i);
+        }
+        assert_eq!(cache.len(), items);
+        let mut drain_collected = cache.drain().collect::<Vec<_>>();
+        assert_eq!(cache.len(), 0);
+        assert_eq!(drain_collected.len(), items);
+        drain_collected.sort();
+        for (i, v) in drain_collected.into_iter().enumerate() {
+            assert_eq!((i, i), v);
+        }
+    }
 }
